@@ -234,7 +234,7 @@ class P4Client(object):
                         if not complete_file_path in files:
                             all_files.append(complete_file_path)
             else:
-                all_files = [file for file in os.listdir(folder) if os.isfile(os.join(folder, file))]
+                all_files = [os.path.join(folder, file) for file in os.listdir(folder) if os.path.isfile(os.path.join(folder, file))]
 
             return self.files_to_p4files(all_files)
 
@@ -282,9 +282,7 @@ class P4Client(object):
         :return: *list* of info dictionaries
         """
         file_list = convert_to_list(file_list) if not isinstance(file_list, list) else file_list
-
         changelist = self.__ensure_changelist(changelist)
-
         info_dicts = self.run_cmd2("reopen", ["-c", changelist] + file_list)
 
         for info_dict in info_dicts:
@@ -369,17 +367,22 @@ class P4Client(object):
         info_dicts = self.run_cmd2("sync", [] + cleaned_folder_list)
         return info_dicts
 
-    def sync_files(self, file_list, verify=True, force=False):
+    def sync_files(self, file_list, revision=-1, verify=True, force=False):
         """
         Syncs files
 
         :param file_list: *list*
         :param verify: *bool* if true, checks that file synced files exist on disk. Throws a warning if they don't
         This could happen when a synced file is deleted locally
+        :param revision: *int* if -1, get latest, else get revision number
         :param force: *bool* force sync
         :return: *list* of info dicts
         """
         file_list = convert_to_list(file_list) if not isinstance(file_list, list) else file_list
+        if revision != -1:
+            verify = False
+            file_list = [f"{path}#{revision}" for path in file_list]
+
         initial_arg_list = ["-f"] if force else []
         if not self.silent: self.__validate_file_list(file_list)
 
@@ -389,8 +392,7 @@ class P4Client(object):
             local_file_paths = self.get_local_paths(file_list)
             for local_file_path in local_file_paths:
                 if not os.path.isfile(local_file_path):
-                    logging.warning("File didn't exist after syncing, try force syncing it instead: %s" %
-                                    local_file_path)
+                    logging.warning(f"File didn't exist after syncing, try force syncing it instead: {local_file_path}")
 
         return info_dicts
 
@@ -437,6 +439,29 @@ class P4Client(object):
 
         return files_and_cl
 
+    def add_or_edit_folders(self, folders, include_subfolders=True, changelist="default"):
+        """
+        Marks a folder for add or edit
+
+        :param folders: *list* or *string*
+        :param include_subfolders: *bool*
+        :param changelist: *string* or *int* changelist number or description. Will be made if it doesn't exist.*string* or *int* changelist number
+        """
+        folders = convert_to_list(folders) if not isinstance(folders, list) else folders
+
+        all_files = []
+        for folder in folders:
+            if include_subfolders:
+                for root, dirs, files in os.walk(folder):
+                    for file_name in files:
+                        complete_file_path = os.path.join(root, file_name)
+                        if not complete_file_path in files:
+                            all_files.append(complete_file_path)
+            else:
+                all_files.extend([os.path.join(folder, file) for file in os.listdir(folder) if os.path.isfile(os.path.join(folder, file))])
+
+        self.add_or_edit_files(all_files, changelist=changelist)
+
     def add_or_edit_files(self, file_list, changelist="default"):
         """
         Marks the files in file_list for add if they are new, or edit if they are already versioned
@@ -852,7 +877,6 @@ def decode_dictionaries(info_dicts):
         decoded_dicts.append(decoded_dict)
     return decoded_dicts
 
-
 def convert_to_list(value):
     if isinstance(value, tuple):
         converted = [v for v in value]
